﻿namespace ProjectSeaplane.CollectionGenericObjects;

/// <summary>
/// Интерфейс описания действий для набора хранимых объектов
/// </summary>
/// <typeparam name="T">Параметр: ограничение - ссылочный тип</typeparam>
public interface ICollectionGenericObjects<T>
    where T : class
{
    /// <summary>
    /// Количество объектов в коллекции
    /// </summary>
    int Count { get; }
    /// <summary>
	/// Установка максимального количества элементов
	/// </summary>
	int MaxCount { get; set; }
    /// <summary>
    /// Добавление объекта в коллекцию
    /// </summary>
    /// <param name="obj">Добавляемый объект</param>
    /// <returns>true - вставка прошла удачно, false - вставка не удалась</returns>
    int Insert(T obj, IEqualityComparer<T?>? compaper = null);
    /// <summary>
    /// Добавление объекта в коллекцию на конкретную позицию
    /// </summary>
    /// <param name="obj">Добавляемый объект</param>
    /// <param name="position">Позиция</param>
    /// <returns>true - вставка прошла удачно, false - вставка не удалась</returns>
    int Insert(T obj, int position, IEqualityComparer<T?>? compaper = null);
    /// <summary>
    /// Удаление объекта из коллекции с конкретной позиции
    /// </summary>
    /// <param name="position">Позиция</param>
    /// <returns>true - удаление прошло удачно, false - удаление не удалось</returns>
    T Remove(int position);
    /// <summary>
    /// Получение объекта по позиции
    /// </summary>
    /// <param name="position">Позиция</param>
    /// <returns>Объект</returns>
    T? Get(int position);
    /// <summary>
    /// получение типа коллекции
    /// </summary>
    CollectionType GetCollectionType { get; }
    /// <summary>
    /// получение объектов коллекции по одному
    /// </summary>
    /// <returns></returns>
    IEnumerable<T?> GetItems();
    /// <summary>
    /// Сортировка коллекции
    /// </summary>
    /// <param name="comparer">Сравнитель объектов</param>
    void CollectionSort(IComparer<T?> comparer);
}