﻿
using ProjectSeaplane.Drawnings;
using ProjectSeaplane.Exceptions;

namespace ProjectSeaplane.CollectionGenericObjects;

/// <summary>
/// Параметризованный набор объектов
/// </summary>
/// <typeparam name="T">Параметр: ограничение - ссылочный тип</typeparam>
public class MassiveGenericObjects<T> : ICollectionGenericObjects<T>
    where T : class
{
    /// <summary>
    /// массив объектов, которые храним
    /// </summary>
    private T?[] _collection;
    public int Count => _collection.Length;

    public int MaxCount
    {
        get
        {
            return _collection.Length;
        }

        set
        {
            if (value > 0)
            {
                if (_collection.Length > 0)
                {
                    Array.Resize(ref _collection, value);
                }
                else
                {
                    _collection = new T?[value];
                }
            }
        }
    }
    public CollectionType GetCollectionType => CollectionType.Massive;

    /// <summary>
    /// Конструктор
    /// </summary>
    public MassiveGenericObjects()
    {
        _collection = Array.Empty<T?>();
    }

    public T? Get(int position)
    {
        if (position >= _collection.Length || position < 0) throw new PositionOutOfCollectionException(position);
        if (_collection[position] == null)                  throw new ObjectNotFoundException(position);
        return _collection[position];
    }

    public int Insert(T obj, IEqualityComparer<T?>? compaper = null)
    {
        // TODO вставка в свободное место набора
        if (compaper != null)
        {
            foreach (T? item in _collection)
            {
                if ((compaper as IEqualityComparer<DrawingBasicSeaplane>).Equals(obj as DrawingBasicSeaplane, item as DrawingBasicSeaplane))
                    throw new ObjectIsEqualException();
            }
        }
        int index = 0;
        while (index < _collection.Length)
        {
            if (_collection[index] == null)
            {
                _collection[index] = obj;
                return index;
            }

            index++;
        }
        throw new CollectionOverflowException(Count);
    }

    public int Insert(T obj, int position, IEqualityComparer<T?>? compaper = null)
    {
        // TODO проверка позиции
        // TODO проверка, что элемент массива по этой позиции пустой, если нет, то
        //		ищется свободное место после этой позиции и идет вставка туда
        //		если нет после, ищем до
        // TODO вставка
        if (compaper != null)
        {
            foreach (T? item in _collection)
            {
                if ((compaper as IEqualityComparer<DrawingBasicSeaplane>).Equals(obj as DrawingBasicSeaplane, item as DrawingBasicSeaplane))
                    throw new ObjectIsEqualException();
            }
        }
        if (position >= _collection.Length || position < 0)
        {
            throw new PositionOutOfCollectionException(position);
        }        
        if (_collection[position] == null)
        {
            _collection[position] = obj;
            return position;
        }
        int index;
        for (index = position + 1; index < _collection.Length; ++index)
        {
            if (_collection[index] == null)
            {
                _collection[position] = obj;
                return position;
            }
        }
        for (index = position - 1; index >= 0; --index)
        {
            if (_collection[index] == null)
            {
                _collection[position] = obj;
                return position;
            }
        }
        throw new CollectionOverflowException(Count);
    }
    public T Remove(int position)
    {
        // TODO проверка позиции
        // TODO удаление объекта из массива, присвоив элементу массива значение null
        if (position >= _collection.Length || position < 0) throw new PositionOutOfCollectionException(position);
        if (_collection[position] == null)                  throw new ObjectNotFoundException(position);
        T obj = _collection[position];
        _collection[position] = null;
        return obj;
    }
    public IEnumerable<T?> GetItems()
    {
        for (int i = 0; i < _collection.Length; i++)
        {
            yield return _collection[i];
        }
    }
    public void CollectionSort(IComparer<T?> comparer)
    {
        Array.Sort(_collection, comparer);
    }
}