﻿using ProjectSeaplane.Entities;

namespace ProjectSeaplane.Drawnings;
/// <summary>
/// расширение для класса EntitySeaplane
/// </summary>
public static class ExtentionDrawningBasicSeaplane
{
    /// <summary>
    /// Разделитель для записи информации по объекту в файл
    /// </summary>
    private static readonly string _separatorForObject = ":";
    /// <summary>
    /// Создание объекта из строки
    /// </summary>
    /// <param name="info">Строка с данными для создания объекта</param>
    /// <returns>Объект</returns>
    public static DrawingBasicSeaplane? CreateDrawningBasicSeaplane(this string info)
    {
        string[] strs = info.Split(_separatorForObject);
        EntityBasicSeaplane? seaplane = EntitySeaplane.CreateEntitySeaplane(strs);
        if (seaplane != null)
        {
            return new DrawingSeaplane((EntitySeaplane)seaplane);
        }
        seaplane = EntityBasicSeaplane.CreateEntityBasicSeaplane(strs);
        if (seaplane!= null)
        {
            return new DrawingBasicSeaplane(seaplane);
        }
        return null;
    }
    /// <summary>
    /// Получение данных для сохранения в файл
    /// </summary>
    /// <param name="drawningBasicSeaplane">Сохраняемый объект</param>
    /// <returns>Строка с данными по объекту</returns>
    public static string GetDataForSave(this DrawingBasicSeaplane drawningBasicSeaplane)
    {
        string[]? array = drawningBasicSeaplane?.EntityBasicSeaplane?.GetStringRepresentation();
        if (array == null)
        {
            return string.Empty;
        }
        return string.Join(_separatorForObject, array);
    }
}
