﻿namespace ProjectSeaplane.Entities;
/// <summary>
/// Класс-сущность "Простой Гидросамолет"
/// </summary>
public class EntityBasicSeaplane
{ 
    /// <summary>
    /// Скорость
    /// </summary>
    public int Speed { get;set; }
    /// <summary>
    /// Вес
    /// </summary>
    public double Weight { get; set; }
    /// <summary>
    /// Основной цвет
    /// </summary>
    public Color BodyColor { get; set; }
    /// <summary>
    /// Расстояние шага передвижения
    /// </summary>
    public double Step => Speed * 100 / Weight;
    /// <summary>
    /// Метод установки основного цвета
    /// </summary>
    /// <param name="color"></param>
    public void setBodyColor(Color color)
    {
        BodyColor = color;
    }
    /// <summary>
    /// Конструктор сущности
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>

    public EntityBasicSeaplane(int speed, double weight, Color bodyColor)
    {
        Speed = speed;
        Weight = weight;
        BodyColor = bodyColor;
    }
    /// <summary>
    /// Получение строк со значениями свойств объекта класса-сущности
    /// </summary>
    /// <returns></returns>
    public virtual string[] GetStringRepresentation()
    {
        return new[] { nameof(EntityBasicSeaplane), Speed.ToString(), Weight.ToString(), BodyColor.Name };
    }
    /// <summary>
    /// Создание объекта из массива строк
    /// </summary>
    /// <param name="strs"></param>
    /// <returns></returns>
    public static EntityBasicSeaplane? CreateEntityBasicSeaplane(string[] strs)
    {
        if (strs.Length != 4 || strs[0] != nameof(EntityBasicSeaplane))
        {
            return null;
        }
        return new EntityBasicSeaplane(Convert.ToInt32(strs[1]), Convert.ToDouble(strs[2]), Color.FromName(strs[3]));
    }
}

